=== modified file 'drizzled/execute.cc'
--- drizzled/execute.cc	2010-12-30 08:27:31 +0000
+++ drizzled/execute.cc	2011-01-13 22:55:57 +0000
@@ -85,17 +85,24 @@
   if (wait && thread && thread->joinable())
   {
     // We want to make sure that we can be killed
-    boost::this_thread::restore_interruption dl(_session.getThreadInterupt());
-    try {
+    if (_session.getThread())
+    {
+      boost::this_thread::restore_interruption dl(_session.getThreadInterupt());
+
+      try {
+        thread->join();
+      }
+      catch(boost::thread_interrupted const&)
+      {
+        // Just surpress and return the error
+        my_error(drizzled::ER_QUERY_INTERRUPTED, MYF(0));
+        return;
+      }
+    }
+    else
+    {
       thread->join();
     }
-    catch(boost::thread_interrupted const&)
-    {
-      // Just surpress and return the error
-      my_error(drizzled::ER_QUERY_INTERRUPTED, MYF(0));
-
-      return;
-    }
   }
 }
 

=== modified file 'drizzled/main.cc'
--- drizzled/main.cc	2011-01-18 17:18:03 +0000
+++ drizzled/main.cc	2011-01-24 00:14:21 +0000
@@ -306,10 +306,8 @@
   if (plugin::Listen::setup())
     unireg_abort(1);
 
-
   assert(plugin::num_trx_monitored_objects > 0);
-  if (drizzle_rm_tmp_tables() ||
-      my_tz_init((Session *)0, default_tz_name))
+  if (drizzle_rm_tmp_tables() || my_tz_init((Session *)0, default_tz_name))
   {
     abort_loop= true;
     select_thread_in_use=0;
@@ -317,7 +315,7 @@
 
     (void) unlink(pid_file.file_string().c_str());	// Not needed anymore
 
-    exit(1);
+    unireg_abort(1);
   }
 
   errmsg_printf(ERRMSG_LVL_INFO, _(ER(ER_STARTUP)), internal::my_progname,
@@ -335,6 +333,8 @@
       currentSession().release();
       currentSession().reset(session.get());
       transaction_services.sendStartupEvent(session.get());
+
+      plugin_startup_window(modules, *(session.get()));
     }
   }
 
@@ -382,8 +382,11 @@
   /* Wait until cleanup is done */
   {
     boost::mutex::scoped_lock scopedLock(session::Cache::singleton().mutex());
+
     while (not ready_to_exit)
+    {
       COND_server_end.wait(scopedLock);
+    }
   }
 
   clean_up(1);

=== modified file 'drizzled/module/loader.cc'
--- drizzled/module/loader.cc	2011-01-10 06:17:57 +0000
+++ drizzled/module/loader.cc	2011-01-13 22:55:57 +0000
@@ -397,6 +397,17 @@
   return false;
 }
 
+/*
+  Window of opportunity for plugins to issue any queries with the database up and running but with no user's connected.
+*/
+void plugin_startup_window(module::Registry &registry, drizzled::Session &session)
+{
+  BOOST_FOREACH(plugin::Plugin::map::value_type value, registry.getPluginsMap())
+  {
+    value.second->startup(session);
+  }
+}
+
 class PrunePlugin :
   public unary_function<string, bool>
 {

=== modified file 'drizzled/plugin.h'
--- drizzled/plugin.h	2011-01-13 02:08:31 +0000
+++ drizzled/plugin.h	2011-01-13 23:16:35 +0000
@@ -170,6 +170,7 @@
 extern bool plugin_init(module::Registry &registry,
                         boost::program_options::options_description &long_options);
 extern bool plugin_finalize(module::Registry &registry);
+extern void plugin_startup_window(module::Registry &registry, drizzled::Session &session);
 extern void my_print_help_inc_plugins(option *options);
 extern bool plugin_is_ready(const LEX_STRING *name, int type);
 extern void plugin_sessionvar_init(Session *session);

=== modified file 'drizzled/plugin/plugin.h'
--- drizzled/plugin/plugin.h	2011-01-10 06:17:57 +0000
+++ drizzled/plugin/plugin.h	2011-01-13 22:55:57 +0000
@@ -26,6 +26,9 @@
 
 namespace drizzled
 {
+
+class Session;
+
 namespace module
 {
 class Module;
@@ -66,6 +69,10 @@
   virtual void prime()
   {
   }
+
+  virtual void startup(drizzled::Session &)
+  {
+  }
  
   void activate()
   {

=== modified file 'plugin/schema_engine/schema.cc'
--- plugin/schema_engine/schema.cc	2011-01-17 04:15:23 +0000
+++ plugin/schema_engine/schema.cc	2011-01-24 00:14:21 +0000
@@ -29,6 +29,10 @@
 #include "drizzled/cursor.h"
 #include "drizzled/data_home.h"
 
+#include <drizzled/pthread_globals.h>
+
+#include <drizzled/execute.h>
+
 #include "drizzled/internal/my_sys.h"
 
 #include <fcntl.h>
@@ -68,8 +72,7 @@
 {
   CachedDirectory directory(getDataHomeCatalog().file_string(), CachedDirectory::DIRECTORY);
   CachedDirectory::Entries files= directory.getEntries();
-
-  mutex.lock();
+  boost::unique_lock<boost::shared_mutex> scopedLock(mutex);
 
   for (CachedDirectory::Entries::iterator fileIter= files.begin();
        fileIter != files.end(); fileIter++)
@@ -80,6 +83,7 @@
     if (not entry->filename.compare(GLOBAL_TEMPORARY_EXT))
       continue;
 
+
     identifier::Schema filename(entry->filename);
     if (readSchemaFile(filename, schema_message))
     {
@@ -89,12 +93,15 @@
         schema_cache.insert(make_pair(schema_identifier.getPath(), new message::Schema(schema_message)));
 
       if (ret.second == false)
-     {
+      {
         abort(); // If this has happened, something really bad is going down.
       }
     }
   }
-  mutex.unlock();
+}
+
+void Schema::startup(drizzled::Session &)
+{
 }
 
 void Schema::doGetSchemaIdentifiers(identifier::Schema::vector &set_of_names)
@@ -142,8 +149,8 @@
     return false;
   }
 
-  mutex.lock();
   {
+    boost::unique_lock<boost::shared_mutex> scopedLock(mutex);
     pair<SchemaCache::iterator, bool> ret=
       schema_cache.insert(make_pair(schema_identifier.getPath(), new message::Schema(schema_message)));
 
@@ -153,7 +160,6 @@
       abort(); // If this has happened, something really bad is going down.
     }
   }
-  mutex.unlock();
 
   return true;
 }
@@ -191,9 +197,8 @@
     cerr << dir;
   }
 
-  mutex.lock();
+  boost::unique_lock<boost::shared_mutex> scopedLock(mutex);
   schema_cache.erase(schema_identifier.getPath());
-  mutex.unlock();
 
   return true;
 }
@@ -207,19 +212,16 @@
 
   if (writeSchemaFile(schema_identifier, schema_message))
   {
-    mutex.lock();
+    boost::unique_lock<boost::shared_mutex> scopedLock(mutex);
+    schema_cache.erase(schema_identifier.getPath());
+
+    pair<SchemaCache::iterator, bool> ret=
+      schema_cache.insert(make_pair(schema_identifier.getPath(), new message::Schema(schema_message)));
+
+    if (ret.second == false)
     {
-      schema_cache.erase(schema_identifier.getPath());
-
-      pair<SchemaCache::iterator, bool> ret=
-        schema_cache.insert(make_pair(schema_identifier.getPath(), new message::Schema(schema_message)));
-
-      if (ret.second == false)
-      {
-        abort(); // If this has happened, something really bad is going down.
-      }
+      abort(); // If this has happened, something really bad is going down.
     }
-    mutex.unlock();
   }
 
   return true;

=== modified file 'plugin/schema_engine/schema.h'
--- plugin/schema_engine/schema.h	2011-01-17 04:15:23 +0000
+++ plugin/schema_engine/schema.h	2011-01-24 00:14:21 +0000
@@ -38,6 +38,7 @@
   bool readSchemaFile(const drizzled::identifier::Schema &schema_identifier, drizzled::message::Schema &schema);
 
   void prime();
+  void startup(drizzled::Session &session);
 
   typedef boost::unordered_map<std::string, drizzled::message::schema::shared_ptr> SchemaCache;
   SchemaCache schema_cache;

